local UH = UnderHood
local Module = UH:GetModule( "Bars" )
local L = Module.L
local OO = UH.OO

local UnitMana = UnitMana
local UnitManaMax = UnitManaMax
local UnitPowerType = UnitPowerType
local UnitExists = UnitExists

local Provider = OO:NewClass( "PowerBarProvider", "BarProviderWithUnit" )

function Provider:init( factory )
	super( self, factory, "AceTimer-3.0" )
end

function Provider:UnitChanged( oldUnit )
	super( self, oldUnit )
	
	if self.unit == "player" or self.unit == "pet" then
		self:ScheduleRepeatingTimer( "UpdateValue", 0.2 )
		self:RegisterEvent( "UNIT_DISPLAYPOWER", "UpdateIfUnit" )
	elseif UH.UnitSendEvents[self.unit] then
		self:RegisterEvent( "UNIT_MANA", "UpdateValueIfUnit" )
		self:RegisterEvent( "UNIT_MAXMANA", "UpdateValueIfUnit" )
		self:RegisterEvent( "UNIT_ENERGY", "UpdateValueIfUnit" )
		self:RegisterEvent( "UNIT_MAXENERGY", "UpdateValueIfUnit" )
		self:RegisterEvent( "UNIT_RAGE", "UpdateValueIfUnit" )
		self:RegisterEvent( "UNIT_MAXRAGE", "UpdateValueIfUnit" )
		self:RegisterEvent( "UNIT_FOCUS", "UpdateValueIfUnit" )
		self:RegisterEvent( "UNIT_MAXFOCUS", "UpdateValueIfUnit" )
		self:RegisterEvent( "UNIT_RUNIC_POWER", "UpdateValueIfUnit" )
		self:RegisterEvent( "UNIT_MAXRUNIC_POWER", "UpdateValueIfUnit" )
		self:RegisterEvent( "UNIT_DISPLAYPOWER", "UpdateIfUnit" )
	end
end

function Provider:UpdateColor()
	super( self )
	
	local colors = Module.db.profile.colors
	local unit = self.unit
	local powerType = UnitExists( unit ) and UnitPowerType( unit ) or -1
	local r,g,b

	if powerType == 0 then
		r,g,b = unpack( colors.power.mana )
	elseif powerType == 1 then
		r,g,b = unpack( colors.power.rage )
	elseif powerType == 2 then
		r,g,b = unpack( colors.power.focus )
	elseif powerType == 3 then
		r,g,b = unpack( colors.power.energy )
	elseif powerType == 6 then
		r,g,b = unpack( colors.power.runicPower )
	else
		r,g,b = unpack( colors.unit.unknown )
	end
	
	self:SetColor( r, g, b, 1 )
end

function Provider:UpdateValue()
	super( self )
	
	local unit = self.unit
	local current, max = UnitMana( unit ), UnitManaMax( unit )
	
	current = (max > 0) and (current / max) or 0
	
	self:SetValue( current )
end

local Factory = OO:NewClass( "PowerBarProviderFactory", "BarProviderFactory" )

function Factory:init()
	super( self, "Power", L["Power"] )
end

function Factory:GetDefaultSettings()
	local settings = super( self )
	
	settings.unit = "default"
	
	return settings
end

function Factory:CreateProvider()
	return Provider:new( self )
end

local factory = Factory:new()

Module:RegisterProviderFactory( factory:GetName(), factory )
