local UH = UnderHood
local Module = UnderHood:GetModule( "Icons" )
local L = LibStub( "AceLocale-3.0" ):GetLocale( "UnderHood_Icons" )

local IsResting = IsResting
local UnitAffectingCombat = UnitAffectingCombat
local UnitIsPVPFreeForAll = UnitIsPVPFreeForAll
local UnitFactionGroup = UnitFactionGroup
local UnitIsPVP = UnitIsPVP
local IsPartyLeader = IsPartyLeader
local IsRaidLeader = IsRaidLeader
local IsRaidOfficer = IsRaidOfficer
local GetLootMethod = GetLootMethod
local GetRaidTargetIndex = GetRaidTargetIndex
local GetPetHappiness = GetPetHappiness
local SetRaidTargetIconTexture = SetRaidTargetIconTexture

local validIcons = {
	resting = L["Resting"],
	combat = L["Combat"],
	pvp = L["PvP"],
	leader = L["Leader"],
	ml = L["Master Looter"],
	rt = L["Raid Target"],
	happiness = L["Happiness"],
}

local unitIcons = {
	player = {
		resting = true,
		combat = true,
		pvp = true,
		leader = true,
		ml = true,
		rt = true,
	},
	target = {
		combat = true,
		pvp = true,
		rt = true,
	},
	focus = {
		combat = true,
		pvp = true,
		rt = true,
	},
	targettarget = {
		combat = true,
		pvp = true,
		rt = true,
	},
	pet = {
		combat = true,
		pvp = true,
		rt = true,
		happiness = true,
	},
}

for i = 1, 4 do
	unitIcons["party"..i] = {
		combat = true,
		pvp = true,
		rt = true,
		leader = true,
		ml = true
	}
	unitIcons["partypet"..i] = {
		combat = true,
		pvp = true,
		rt = true,
	}
end

local iconTextures = {
	resting = { "Interface\\CharacterFrame\\UI-StateIcon", 0.09, 0.43, 0.08, 0.42 },
	combat = { "Interface\\CharacterFrame\\UI-StateIcon", 0.57, 0.9, 0.08, 0.41 },
	pvp = {
		{ "Interface\\TargetingFrame\\UI-PVP-FFA", 0.05, 0.605, 0.015, 0.57 },
		{ "Interface\\TargetingFrame\\UI-PVP-Horde", 0.08, 0.58, 0.045, 0.545 },
		{ "Interface\\TargetingFrame\\UI-PVP-Alliance", 0.07, 0.58, 0.06, 0.57 },
	},
	leader = { "Interface\\GroupFrame\\UI-Group-LeaderIcon", 0.1, 0.84, 0.14, 0.88 },
	ml = { "Interface\\GroupFrame\\UI-Group-MasterLooter", 0.15, 0.9, 0.15, 0.9 },
	rt = { "Interface\\TargetingFrame\\UI-RaidTargetingIcons", 0, 1, 0, 1 },
	happiness = {
		{ "Interface\\PetPaperDollFrame\\UI-PetHappiness", 0.375, 0.5625, 0, 0.359375 },
		{ "Interface\\PetPaperDollFrame\\UI-PetHappiness", 0.1875, 0.375, 0, 0.359375 },
		{ "Interface\\PetPaperDollFrame\\UI-PetHappiness", 0, 0.1875, 0, 0.359375 },
	},
}

local IconFrame = UH:RegisterFrameClass( "Icon", L["Icon"], "SecureUnitFrame" )

function IconFrame:init()
	super( self )

	self.rtiForConfig = math.random( 8 )
	
	self.texture = self.frame:CreateTexture()
	self.texture:SetAllPoints( self.frame )
	self.texture:Show()
end

function IconFrame:OnAcquire( settings )
	super( self, settings )
	
	UH:RegisterUpdateTarget( self )
end

function IconFrame:OnRelease()
	UH:UnregisterUpdateTarget( self )
	
	super( self )
end

function IconFrame:GetDefaultSettings()
	local settings = super( self )
	
	settings.positionAndSize.width = 20
	settings.positionAndSize.height = 20
	
	settings.icon = "resting"
	
	return settings
end

function IconFrame:ProcessUpdate()
	local icon = self.settings.icon
	local unit = self:GetUnit()
	local configMode = self.configMode
	local visible, set = true, nil
	local iconIndex

	if icon == "resting" then
		if not IsResting() then visible = false end
	elseif icon == "combat" then
		if not( UnitAffectingCombat( unit ) ) then visible = false end
	elseif icon == "pvp" then
		local pvpt
		
		if UnitIsPVPFreeForAll( unit ) then
			pvpt = 'FFA'
		else
			local faction = UnitFactionGroup( unit )
			
			if faction and UnitIsPVP( unit ) then
				pvpt = faction
			end
		end
		
		if pvpt or configMode then
			if pvpt == "Horde" then
				iconIndex = 2
			elseif pvpt == "Alliance" then
				iconIndex = 3
			else
				iconIndex = 1
			end
		else
			visible = false
		end
	elseif icon == "leader" then
		if not( IsPartyLeader() or IsRaidLeader() or IsRaidOfficer() ) then visible = false end
	elseif icon == "ml" then
		local _, lootmaster = GetLootMethod()
		
		if not( lootmaster == 0 ) then visible = false end
	elseif icon == "rt" then
		local rti = GetRaidTargetIndex( unit )
		
		if not configMode then
			if rti then
				iconIndex = rti
				set = true
			else
				visible = false
			end
		else
			iconIndex = self.rtiForConfig
			set = true
		end
	elseif icon == "happiness" then
		local happiness, damagePercentage, loyaltyRate = GetPetHappiness()

		if happiness == 3 or configMode then
			iconIndex = 3
		elseif happiness == 2 then
			iconIndex = 2
		elseif happiness == 1 then
			iconIndex = 1
		else
			visible = false
		end
	end

	if visible or configMode then
		local tex, l, r, t, b

		if iconIndex and not( set ) then
			tex, l, r, t, b = unpack( iconTextures[icon][iconIndex] )
		else
			tex, l, r, t, b = unpack( iconTextures[icon] )
		end
		
		self.texture:SetTexture( tex )
		self.texture:SetTexCoord( l, r, t, b )
		
		if set then
			SetRaidTargetIconTexture( self.texture, iconIndex )
		end

		if not self.texture:IsVisible() then
			self.texture:Show()
		end
	else
		self.texture:Hide()
	end
end

function IconFrame:UnitChanged( unit )
	super( self, unit )
	
	self:ChangeIcon()
end

function IconFrame:ChangeIcon( icon )
	local unit = self:GetUnit()
	local needRefresh = false
	icon = icon or self.settings.icon
	
	if not unitIcons[unit][icon] then
		for k in pairs( unitIcons[unit] ) do
			icon = k
			needRefresh = true
			break
		end
	end
	
	self.settings.icon = icon
	
	if needRefresh then
		UH:NotifyOptionsChanged()
	end
end

function IconFrame:CreateOptions()
	local options = super( self )
	
	options.general.args.icon = {
		type = "select",
		name = L["Icon"],
		values = function()
			local v = {}
			
			for icon in pairs( unitIcons[self:GetUnit()] ) do
				v[icon] = validIcons[icon]
			end
			
			return v
		end,
		get = function() return self.settings.icon end,
		set = function( info, value ) self:ChangeIcon( value ) end,
	}
	
	return options
end
